@@ -36,25 +36,27 @@ public:
                QWidget * parent = 0, Qt::WindowFlags f = 0);
     ~MainWindow();
 
-    bool dropMode() { return m_dropMode; }
+    bool dropMode() const { return m_dropMode; }
+    void setup_ContextMenu_Actions(QMenu* contextMenu) const;
 
 protected:
      bool event(QEvent* event);
 
 private:
-    QActionGroup *tabPosition, *scrollBarPosition;
-    QMenu *tabPosMenu, *scrollPosMenu;
-
-    QAction *renameSession;
+    QActionGroup *tabPosition, *scrollBarPosition, *keyboardCursorShape;
+    QMenu *tabPosMenu, *scrollPosMenu, *keyboardCursorShapeMenu;
 
     QString m_initWorkDir;
     QString m_initShell;
 
     QDockWidget *m_bookmarksDock;
 
+    void setup_Action(const char *name, QAction *action, const char *defaultShortcut, const QObject *receiver,
+                      const char *slot, QMenu *menu = NULL, const QVariant &data = QVariant());
     void setup_FileMenu_Actions();
     void setup_ActionsMenu_Actions();
     void setup_ViewMenu_Actions();
+    void setupCustomDirs();
 
     void closeEvent(QCloseEvent*);
 
@@ -65,6 +67,9 @@ private:
     void realign();
     void setDropShortcut(QKeySequence dropShortCut);
 
+    bool hasMultipleTabs();
+    bool hasMultipleSubterminals();
+
 private slots:
     void on_consoleTabulator_currentChanged(int);
     void propertiesChanged();
@@ -86,5 +91,8 @@ private slots:
     void bookmarksDock_visibilityChanged(bool visible);
 
     void addNewTab();
+    void onCurrentTitleChanged(int index);
+
+    void aboutToShowActionsMenu();
 };
 #endif //MAINWINDOW_H
