@@ -46,8 +46,7 @@ Properties::~Properties()
 {
     qDebug("Properties destructor called");
     saveSettings();
-    m_settings->deleteLater();
-    delete m_instance;
+    delete m_settings;
     m_instance = 0;
 }
 
@@ -105,11 +104,14 @@ void Properties::loadSettings()
 
     appTransparency = m_settings->value("MainWindow/ApplicationTransparency", 0).toInt();
     termTransparency = m_settings->value("TerminalTransparency", 0).toInt();
+    backgroundImage = m_settings->value("TerminalBackgroundImage", QString()).toString();
 
     /* default to Right. see qtermwidget.h */
     scrollBarPos = m_settings->value("ScrollbarPosition", 2).toInt();
     /* default to North. I'd prefer South but North is standard (they say) */
     tabsPos = m_settings->value("TabsPosition", 0).toInt();
+    /* default to BlockCursor */
+    keyboardCursorShape = m_settings->value("KeyboardCursorShape", 0).toInt();
     hideTabBarWithOneTab = m_settings->value("HideTabBarWithOneTab", false).toBool();
     m_motionAfterPaste = m_settings->value("MotionAfterPaste", 0).toInt();
 
@@ -136,6 +138,9 @@ void Properties::loadSettings()
     dropWidht = m_settings->value("Width", 70).toInt();
     dropHeight = m_settings->value("Height", 45).toInt();
     m_settings->endGroup();
+
+    changeWindowTitle = m_settings->value("ChangeWindowTitle", true).toBool();
+    changeWindowIcon = m_settings->value("ChangeWindowIcon", true).toBool();
 }
 
 void Properties::saveSettings()
@@ -150,8 +155,10 @@ void Properties::saveSettings()
     while( it.hasNext() )
     {
         it.next();
-        QKeySequence shortcut = it.value()->shortcut();
-        m_settings->setValue( it.key(), shortcut.toString() );
+        QStringList sequenceStrings;
+        foreach (QKeySequence shortcut, it.value()->shortcuts())
+            sequenceStrings.append(shortcut.toString());
+        m_settings->setValue(it.key(), sequenceStrings.join('|'));
     }
     m_settings->endGroup();
 
@@ -180,8 +187,10 @@ void Properties::saveSettings()
 
     m_settings->setValue("MainWindow/ApplicationTransparency", appTransparency);
     m_settings->setValue("TerminalTransparency", termTransparency);
+    m_settings->setValue("TerminalBackgroundImage", backgroundImage);
     m_settings->setValue("ScrollbarPosition", scrollBarPos);
     m_settings->setValue("TabsPosition", tabsPos);
+    m_settings->setValue("KeyboardCursorShape", keyboardCursorShape);
     m_settings->setValue("HideTabBarWithOneTab", hideTabBarWithOneTab);
     m_settings->setValue("MotionAfterPaste", m_motionAfterPaste);
     m_settings->setValue("Borderless", borderless);
@@ -207,6 +216,8 @@ void Properties::saveSettings()
     m_settings->setValue("Height", dropHeight);
     m_settings->endGroup();
 
+    m_settings->setValue("ChangeWindowTitle", changeWindowTitle);
+    m_settings->setValue("ChangeWindowIcon", changeWindowIcon);
 }
 
 void Properties::migrate_settings()
