@@ -28,6 +28,7 @@
 
 
 #define TAB_INDEX_PROPERTY "tab_index"
+#define TAB_CUSTOM_NAME_PROPERTY "custom_name"
 
 
 TabWidget::TabWidget(QWidget* parent) : QTabWidget(parent), tabNumerator(0)
@@ -78,10 +79,14 @@ int TabWidget::addNewTab(const QString & shell_program)
     }
 
     TermWidgetHolder *console = new TermWidgetHolder(cwd, shell_program, this);
+    console->setWindowTitle(label);
     connect(console, SIGNAL(finished()), SLOT(removeFinished()));
     connect(console, SIGNAL(lastTerminalClosed()), this, SLOT(removeFinished()));
+    connect(console, &TermWidgetHolder::termTitleChanged, this, &TabWidget::onTermTitleChanged);
+    connect(this, &QTabWidget::currentChanged, this, &TabWidget::currentTitleChanged);
 
     int index = addTab(console, label);
+    console->setProperty(TAB_CUSTOM_NAME_PROPERTY, false);
     updateTabIndices();
     setCurrentIndex(index);
     console->setInitialFocus();
@@ -152,6 +157,21 @@ void TabWidget::updateTabIndices()
         widget(i)->setProperty(TAB_INDEX_PROPERTY, i);
 }
 
+void TabWidget::onTermTitleChanged(QString title, QString icon)
+{
+    TermWidgetHolder * console = qobject_cast<TermWidgetHolder*>(sender());
+    const bool custom_name = console->property(TAB_CUSTOM_NAME_PROPERTY).toBool();
+    if (!custom_name)
+    {
+        const int index = console->property(TAB_INDEX_PROPERTY).toInt();
+
+        setTabIcon(index, QIcon::fromTheme(icon));
+        setTabText(index, title);
+        if (currentIndex() == index)
+            emit currentTitleChanged(index);
+    }
+}
+
 void TabWidget::renameSession(int index)
 {
     bool ok = false;
@@ -160,10 +180,19 @@ void TabWidget::renameSession(int index)
                                         QString(), &ok);
     if(ok && !text.isEmpty())
     {
+        setTabIcon(index, QIcon{});
         setTabText(index, text);
+        widget(index)->setProperty(TAB_CUSTOM_NAME_PROPERTY, true);
+        if (currentIndex() == index)
+            emit currentTitleChanged(index);
     }
 }
 
+void TabWidget::renameCurrentSession()
+{
+    renameSession(currentIndex());
+}
+
 void TabWidget::renameTabsAfterRemove()
 {
 // it breaks custom names - it replaces original/custom title with shell no #
@@ -174,13 +203,14 @@ void TabWidget::renameTabsAfterRemove()
 #endif
 }
 
-void TabWidget::contextMenuEvent ( QContextMenuEvent * event )
+void TabWidget::contextMenuEvent(QContextMenuEvent *event)
 {
     QMenu menu(this);
 
     QAction *close = menu.addAction(QIcon::fromTheme("document-close"), tr("Close session"));
-    QAction *rename = menu.addAction(tr("Rename session"));
-    rename->setShortcut(tr(RENAME_SESSION_SHORTCUT));
+    QAction *rename = menu.addAction(Properties::Instance()->actions[RENAME_SESSION]->text());
+    rename->setShortcut(Properties::Instance()->actions[RENAME_SESSION]->shortcut());
+    rename->blockSignals(true);
 
     int tabIndex = tabBar()->tabAt(event->pos());
     QAction *action = menu.exec(event->globalPos());
@@ -223,24 +253,25 @@ void TabWidget::removeFinished()
 
 void TabWidget::removeTab(int index)
 {
-    setUpdatesEnabled(false);
+    if (count() > 1) {
+        setUpdatesEnabled(false);
 
-    QWidget * w = widget(index);
-    QTabWidget::removeTab(index);
-    w->deleteLater();
+        QWidget * w = widget(index);
+        QTabWidget::removeTab(index);
+        w->deleteLater();
 
-    updateTabIndices();
-    int current = currentIndex();
-    if (current >= 0 )
-    {
-        qobject_cast<TermWidgetHolder*>(widget(current))->setInitialFocus();
-    }
-// do not decrease it as renaming is disabled in renameTabsAfterRemove
-//    tabNumerator--;
-    setUpdatesEnabled(true);
-
-    if (count() == 0)
+        updateTabIndices();
+        int current = currentIndex();
+        if (current >= 0 )
+        {
+            qobject_cast<TermWidgetHolder*>(widget(current))->setInitialFocus();
+        }
+    // do not decrease it as renaming is disabled in renameTabsAfterRemove
+    //    tabNumerator--;
+        setUpdatesEnabled(true);
+    } else {
         emit closeTabNotification();
+    }
 
     renameTabsAfterRemove();
     showHideTabBar();
@@ -332,7 +363,6 @@ void TabWidget::changeScrollPosition(QAction *triggered)
     if(!scrollPosition)
         qFatal("scrollPosition is NULL");
 
-
     Properties::Instance()->scrollBarPos =
             scrollPosition->actions().indexOf(triggered);
 
@@ -353,7 +383,19 @@ void TabWidget::changeTabPosition(QAction *triggered)
     setTabPosition(position);
     prop->tabsPos = position;
     prop->saveSettings();
-    return;
+}
+
+void TabWidget::changeKeyboardCursorShape(QAction *triggered)
+{
+    QActionGroup *keyboardCursorShape = static_cast<QActionGroup *>(sender());
+    if(!keyboardCursorShape)
+        qFatal("keyboardCursorShape is NULL");
+
+    Properties::Instance()->keyboardCursorShape =
+            keyboardCursorShape->actions().indexOf(triggered);
+
+    Properties::Instance()->saveSettings();
+    propertiesChanged();
 }
 
 void TabWidget::propertiesChanged()
