@@ -49,6 +49,7 @@ public slots:
     void moveLeft();
     void moveRight();
     void renameSession(int);
+    void renameCurrentSession();
     void setWorkDirectory(const QString&);
 
     void switchNextSubterminal();
@@ -66,6 +67,7 @@ public slots:
 
     void changeTabPosition(QAction *);
     void changeScrollPosition(QAction *);
+    void changeKeyboardCursorShape(QAction *);
     void propertiesChanged();
 
     void clearActiveTerminal();
@@ -80,6 +82,7 @@ public slots:
 signals:
     void closeTabNotification();
     void tabRenameRequested(int);
+    void currentTitleChanged(int);
 
 protected:
     enum Direction{Left = 1, Right};
@@ -93,6 +96,7 @@ protected:
     bool eventFilter(QObject *obj, QEvent *event);
 protected slots:
     void updateTabIndices();
+    void onTermTitleChanged(QString title, QString icon);
 
 private:
     int tabNumerator;
